<?php
require __DIR__ . '/head.php';
require __DIR__ . '/header.php';

$controllerCidade = new ControllerCidade();

$cidade_selecionada = isset($_POST['cidade']) ? $_POST['cidade'] : null;
$cidade_nome = null;
$busca = isset($_POST['busca']) ? $_POST['busca'] : null;
$filtro = array('status' => 'CONFIRMADA');
if ($busca || $cidade_selecionada) {

    $lista = $controllerGaleria->entityManager->getRepository("Nascente")
            ->createQueryBuilder('n');
    $sql = 'n.status LIKE :status';

    if($busca){
        $sql .= ' AND (n.observacao LIKE :busca OR n.titulo LIKE :busca OR n.descricao LIKE :busca OR n.corrego LIKE :busca OR n.microbacia LIKE :busca)';
        $lista->setParameter('busca', "%{$busca}%" );
    }
    
    if( $cidade_selecionada ){
        $lista->innerJoin('Localizacao', 'l', 'WITH', 'n.localizacao = l.id')
            ->innerJoin('Cidade', 'c', 'WITH', 'l.cidade = c.id')
            ->setParameter('cidade', $cidade_selecionada )
            ->where('c.id = :cidade');
            $sql .= ' AND c.id = :cidade';

        $cidade_nome = $controllerCidade->getById($cidade_selecionada);
        $cidade_nome = $cidade_nome->getNome();
    }
    $lista->where($sql);

    $lista->setParameter('status', 'CONFIRMADA' );

    $lista = $lista->orderBy('n.numero', 'ASC')->getQuery()->getResult();

}else{
    $lista = $controllerNascente->findBy($filtro, array('numero' => 'ASC'));
}

$cidades = $controllerCidade->findBy(array('status' => ['ATIVO']), array('nome' => 'ASC'));

$slug = 'nascentes';

?>
<!-- Contact -->
<div class="page-header py-3">
    <h1 class="page-title m-0 p-2 text-center text-uppercase font-weight-bold">Localização e características gerais</h1>
</div>

<div class="page-content py-5">
    <div class="container">
        <div class="row">
            <div class="col-lg-9">
                <form method="post">
                    <div class="row justify-content-end">
                        <?php echo $cidade_nome ? "<div class=\"col-lg-auto\" style=\"margin-top:10px;\">Cidade: <span class=\"text-blue\">{$cidade_nome}</span></div>" : ''; ?>
                        <?php echo $busca ? "<div class=\"col-lg-auto\" style=\"margin-top:10px;\">Busca por: <span class=\"text-blue\">{$busca}</span></div>" : ''; ?>
                        <div class="col-6 col-lg-auto" >
                            <select name="cidade" class="custom-select shadow-sm">
                                <option value="">Todas Cidades</option>
                                <?php foreach ($cidades as $k => $cidade): ?>
                                <option value="<?php echo $cidade->getId(); ?>" <?php echo ($cidade_selecionada == $cidade->getId()) ? 'selected="selected"' : ''; ?> ><?php echo $cidade->getNome(); ?></option>
                                <?php endforeach;?>
                            </select>
                        </div>
                        <div class="col-6 col-lg-auto">
                            <div class="input-group shadow-sm">
                                <input type="text" name="busca" class="form-control" placeholder="Buscar em nascentes" value="<?php echo $busca; ?>">
                                <div class="input-group-append">
                                    <button type="submit" class="btn bg-blue">
                                        <i class="fas fa-search text-white"></i>
                                    </button>
                                </div>
                            </div>
                        </div>
                    </div>
                </form>
                <hr>
                <div class="p-2 mb-3 bg-white shadow-sm rounded">
                    <div id="map" style="height: 790px;"></div>
                </div>
                <div class="fountains" style="height: 340px; overflow-y: scroll; overflow-x: hidden;">
                    <div class="row">
                        <?php foreach ($lista as $k => $nascente) {?>
                        <div class="col-lg-4">
                            <div class="fountain mb-3 p-2 bg-white shadow-sm">
                                <h5 class="fountain-name mb-2 text-blue"><?php echo $nascente->getTitulo(); ?></h5>
                                <a class="btn btn-link p-0" href="nascente/<?php echo $nascente->getSlug(); ?>">Clique a para mais informações</a>
                            </div>
                        </div>
                        <?php }?>
                    </div>
                </div>
            </div>
            <?php require __DIR__ . '/sidebar-nascentes.php';?>
        </div>
    </div>
</div>

<script>
    // Style do mapa
    function initialize()
    {
        var infowindow = new google.maps.InfoWindow();

        var icon = {
            url: 'assets/img/pin.png', // url
            scaledSize: new google.maps.Size(23, 30), // scaled size
            origin: new google.maps.Point(0,0), // origin
            anchor: new google.maps.Point(0, 0) // anchor
        };

        var map = new google.maps.Map(document.getElementById('map'), {
            center: new google.maps.LatLng(<?php echo $latitude_padrao; ?>, <?php echo $longitude_padrao;?>),
            zoom: 10,
            scrollwheel: false,
            styles: [{"featureType": "all", "stylers": [{"saturation": 0 }, {"hue": "#e7ecf0"} ] }, {"featureType": "road", "stylers": [{"saturation": -70 } ] }, {"featureType": "transit", "stylers": [{"visibility": "off"} ] }, {"featureType": "poi", "stylers": [{"visibility": "off"} ] }, {"featureType": "water", "stylers": [{"visibility": "simplified"}, {"saturation": -60 } ] } ]
        });

        var bounds     = new google.maps.LatLngBounds();

        <?php foreach ($lista as $k => $nascente):
	if ($nascente->getLocalizacao()->getLatitude() && $nascente->getLocalizacao()->getLongitude()):
		$latitude = str_replace(',', '.', $nascente->getLocalizacao()->getLatitude());
		$longitude = str_replace(',', '.', $nascente->getLocalizacao()->getLongitude());

		$cidade = $nascente->getLocalizacao()->getCidade() ? $nascente->getLocalizacao()->getCidade()->getNome() : '';
		$estado = $nascente->getLocalizacao()->getCidade() ? $nascente->getLocalizacao()->getCidade()->getEstado() : '';
		?>
		    var latLng<?php echo $k; ?> = new google.maps.LatLng(<?php echo $latitude; ?>, <?php echo $longitude; ?>);

		  var contString<?php echo $k; ?> = '<div id="content">'+
		      '<h5 class="text-blue"><?php echo $nascente->getTitulo(); ?></h5>'+
		        '<p class="mb-2">Localização: <?php echo $nascente->getLocalizacao()->getBairro(); ?>, <?php echo $cidade; ?>, <?php echo $estado; ?></p>'+
		        '<p class="mb-2">Córrego: <?php echo $nascente->getCorrego(); ?></p>'+
		        '<p class="mb-2 text-muted">Coordenadas: <?php echo $latitude; ?>, <?php echo $longitude; ?></p>'+
		        '<p class="mb-2"><a href="nascente/<?php echo $nascente->getSlug(); ?>">Saiba Mais</a></p>'+
		      '</div>';
		  var infowindow<?php echo $k; ?> = new google.maps.InfoWindow({
		            content: contString<?php echo $k; ?>});

		      var marker<?php echo $k; ?> = new google.maps.Marker({
		        map: map,
		        icon: icon,
		        position: latLng<?php echo $k; ?>,
		        title: '<?php echo $nascente->getTitulo(); ?>'
		      });

		      marker<?php echo $k; ?>.addListener('click', function() {
		        infowindow<?php echo $k; ?>.open(map, marker<?php echo $k; ?>);
		        map.panTo(latLng<?php echo $k; ?>);

		    });
                bounds.extend(latLng<?php echo $k; ?>);
		    <?php endif;endforeach;?>

            map.fitBounds(bounds);

    }

    function loadScript()
    {
        var script = document.createElement('script');
        script.type = 'text/javascript';
        script.src = 'https://maps.googleapis.com/maps/api/js?callback=initialize&key=<?php echo $google_api; ?>';
        document.body.appendChild(script);
    }

    window.onload = loadScript;
</script>

<?php require __DIR__ . '/footer.php';?>
<?php require __DIR__ . '/foot.php';?>

