<?php

/**
 * @Company: NetPixel Tecnologia
 * @Author: fhagnerduarte
 * @Date:   2018-02-21 22:42:30
 * @Last Modified by:   Fhagner Duarte
 * @Last Modified time: 2019-02-23 13:09:26
 */

namespace Framework;

use Doctrine\ORM\EntityManager;
use Doctrine\ORM\Tools\Setup;

class Banco {

	private $entidades = null;
	private $isDevMode = null;
	private $dbParams = null;
	public $status = array();
	public $entityManager = null;
	public $tabela = '';

	public function __construct($tabela = null) {
		try {

			$this->tabela = $tabela;
			$this->entidades = array();
			$this->isDevMode = getenv("DB_DEVMODE");
			$this->dbParams = array(
				'host' => getenv("DB_HOST"),
				'charset' => getenv("DB_CHARSET"),
				'port' => getenv("DB_PORT"),
				'driver' => getenv("DB_DRIVE"),
				'user' => getenv("DB_USERNAME"),
				'password' => getenv("DB_PASSWORD"),
				'dbname' => getenv("DB_DATABASE"),
			);

			$config = Setup::createAnnotationMetadataConfiguration($this->entidades, $this->isDevMode, __DIR__ . '/../proxy/');

			$config->setAutoGenerateProxyClasses(true);

			$eventManager = new \Doctrine\Common\EventManager();
			$eventManager->addEventSubscriber(new \LogEventListener());

			$this->entityManager = EntityManager::create($this->dbParams, $config, $eventManager);

			$this->status = array(
				'EXCLUIDO' => 'excluído',
				'ATIVO' => 'ativado',
				'INATIVO' => 'desativado');
		} catch (Exception $e) {
			throw new Exception($e->getMessage(), 1);
		}
	}

	public function merge($obj) {
		$this->entityManager->getConnection()->beginTransaction();
		try {
			$this->entityManager->merge($obj);
			$this->entityManager->flush();
			$this->entityManager->getConnection()->commit();
		} catch (Exception $e) {
			$this->entityManager->getConnection()->rollBack();

			throw new Exception($e->getMessage(), 1);
		}
	}

	public function mergeRefresh($obj) {
		$this->entityManager->getConnection()->beginTransaction();
		try {
			$this->entityManager->merge($obj);
			$this->entityManager->flush();
			$this->entityManager->refresh($obj);
			$this->entityManager->getConnection()->commit();
		} catch (Exception $e) {
			$this->entityManager->getConnection()->rollBack();

			throw new Exception($e->getMessage(), 1);
		}
	}

	public function persist($obj) {
		$this->entityManager->getConnection()->beginTransaction();
		try {
			$this->entityManager->persist($obj);
			$this->entityManager->flush();
			$this->entityManager->getConnection()->commit();
			return $obj;
		} catch (Exception $e) {
			$this->entityManager->getConnection()->rollBack();
			throw new Exception($e->getMessage(), 1);
		}
	}

	public function delete($obj) {
		$this->entityManager->getConnection()->beginTransaction();
		try {
			$this->entityManager->remove($obj);
			$this->entityManager->flush();
			$this->entityManager->getConnection()->commit();
		} catch (Exception $e) {
			$this->entityManager->getConnection()->rollBack();

			throw new Exception($e->getMessage(), 1);
		}
	}

	public function getById($id) {
		try {
			return $this->entityManager
				->getRepository($this->tabela)
				->findOneBy(array('id' => $id));
		} catch (Exception $e) {

			throw new Exception($e->getMessage(), 1);
		}
	}

	public function getByAttr($attr = array()) {
		try {
			return $this->entityManager
				->getRepository($this->tabela)
				->findOneBy($attr);
		} catch (Exception $e) {
			debug_print_backtrace();
			throw new Exception($e->getMessage(), 1);
		}
	}

	public function findBy($attr = array(), $order = array(), $limit = null, $offset = null) {
		try {

			return $this->entityManager
				->getRepository($this->tabela)
				->findBy($attr, $order, $limit, $offset);
		} catch (Exception $e) {

			throw new Exception($e->getMessage(), 1);
		}
	}

	public function selectAllbyStatus($limite = 0, $status = null) {

		try {

			if ($status) {
				$sql = "SELECT obj FROM {$this->tabela} obj WHERE obj.status = '{$status}' ORDER BY obj.id DESC ";
			} else {
				$sql = "SELECT obj FROM {$this->tabela} obj WHERE obj.status <> 'EXCLUIDO' ORDER BY obj.id DESC ";
			}

			$query = $this->entityManager->createQuery($sql);
			if (is_array($limite)) {
				$query->setFirstResult($limite[0])->setMaxResults($limite[1]);
			} else {
				if ($limite > 0) {
					$query->setMaxResults($limite);
				}
			}
			$lista = $query->getResult();

			return $lista;
		} catch (Exception $e) {

			throw new Exception($e->getMessage(), 1);
		}
	}

	public function selectAll($limite = 0) {

		try {

			$sql = "SELECT obj FROM {$this->tabela} obj ORDER BY obj.id DESC ";

			$query = $this->entityManager->createQuery($sql);
			if (is_array($limite)) {
				$query->setFirstResult($limite[0])->setMaxResults($limite[1]);
			} else {
				if ($limite > 0) {
					$query->setMaxResults($limite);
				}
			}
			$lista = $query->getResult();

			return $lista;
		} catch (Exception $e) {

			throw new Exception($e->getMessage(), 1);
		}
	}

	public function query($sql) {
		try {
			$query = $this->entityManager->getConnection()->prepare($sql);
			$query->execute();

			return $query->fetchAll();
		} catch (Exception $e) {

			throw new Exception($e->getMessage(), 1);
		}
	}

	public function execute($sql) {
		try {
			$query = $this->entityManager->getConnection()->prepare($sql);
			$query->execute();

		} catch (Exception $e) {

			throw new Exception($e->getMessage(), 1);
		}
	}

	public function criarSlug($campo_titulo, $titulo, $slug = null, $id = null) {

		try {
			$helper = new \Framework\Helper();

			$tabela = strtolower($this->tabela);
			if (empty($id)) {
				$sql = "SELECT COUNT({$tabela}_id) AS total FROM {$tabela} WHERE {$tabela}_{$campo_titulo} = :titulo ";
			} else {
				$sql = "SELECT COUNT({$tabela}_id) AS total FROM {$tabela} WHERE {$tabela}_slug = :slug AND {$tabela}_id <> :id ";
			}

			$stmt = $this->entityManager->getConnection()->prepare($sql);

			if (empty($id)) {
				$stmt->bindValue("titulo", $titulo);
			} else {
				$stmt->bindValue("slug", $slug);
				$stmt->bindValue("id", $id);
			}
			$stmt->execute();
			$lista = $stmt->fetchAll();

			$slug = $helper->createSlug($titulo);
			if (isset($lista[0]['total']) && $lista[0]['total'] > 0) {
				$slug .= '-' . ($lista[0]['total'] + 1);
			}

			return $slug;
		} catch (Exception $e) {

			throw new Exception($e->getMessage(), 1);
		}
	}

	/**
	 * @TODO: refatorar a funcao para eviar a duplicidade
	 */
	public function criarSlugAlternatica($campo_titulo, $titulo, $slug = null, $id = null) {

		try {
			$helper = new \Framework\Helper();

			$tabela = strtolower(preg_replace('/(?<!^)[A-Z]/', '_$0', $this->tabela));
			if (empty($id)) {
				$sql = "SELECT COUNT(id) AS total FROM {$tabela} WHERE {$campo_titulo} = :titulo ";
			} else {
				$sql = "SELECT COUNT(id) AS total FROM {$tabela} WHERE slug = :slug AND id <> :id ";
			}

			$stmt = $this->entityManager->getConnection()->prepare($sql);

			if (empty($id)) {
				$stmt->bindValue("titulo", $titulo);
			} else {
				$stmt->bindValue("slug", $slug);
				$stmt->bindValue("id", $id);
			}
			$stmt->execute();
			$lista = $stmt->fetchAll();

			$slug = $helper->createSlug($titulo);
			if (isset($lista[0]['total']) && $lista[0]['total'] > 0) {
				$slug .= '-' . ($lista[0]['total'] + 1);
			}

			return $slug;
		} catch (Exception $e) {

			throw new Exception($e->getMessage(), 1);
		}
	}

	public function changeStatus($id, $action) {

		try {
			$obj = $this->getByAttr(array('id' => $id));
			$obj->setStatus($action);

			$this->merge($obj);

			return 'Item ' . $this->status[$action];
		} catch (Exception $e) {

			throw new Exception($e->getMessage(), 1);
		}
	}

}
