<?php

/**
 * @Company: NetPixel Tecnologia
 * @Author: fhagnerduarte
 * @Date:   2018-02-21 22:42:12
 * @Last Modified by:   fhagnerduarte
 * @Last Modified time: 2018-02-28 21:09:10
 */

namespace Framework;

class Module
{

    private $moduleFolder = array();
    private $modules = array();
    private $modulesName = array();
    private $patch = 'modules/';

    public function __construct($patch = 'modules/')
    {
        $this->patch = $patch;
    }

    public function scanModules()
    {

        $modules = array();
        $directory = opendir($this->patch);
        while ($file = readdir($directory)) {
            if (($file != '.') && ($file != '..')) {
                $this->moduleFolder[] = $file;
                $this->processXmlModule($file);
            }
        }
        closedir($directory);

        return $this->modules;
    }

    private function processXmlModule($module)
    {
        $file = $this->patch . $module . '/module.xml';

        if (!file_exists($file)) {
            return false;
        }

        $xml = simplexml_load_file($file);

        $menu_itens = array();
        foreach ($xml->menuItens->children() as $child) {
            $attributes = $child->attributes();
            $menu_itens[(string) $attributes['name']] = ($attributes['label']);
        }

        $panel_itens = array();
        foreach ($xml->panelItens->children() as $child) {
            $attributes = $child->attributes();
            // $panel_itens[(string) $attributes['name']] = ($attributes['label']);
            $panel_itens[] = array(
                'page' => (string) $attributes['name'],
                'label' => ($attributes['label']),
                'icon' => ($attributes['icon']));
        }

        $module_name = (string) $xml->name;

        $this->modulesName[] = $module_name;

        $this->modules[$module_name] = array(
            'name' => $module_name,
            'label' => ($xml->label),
            'version' => $xml->version,
            'patch' => $xml->patch,
            'menu_itens' => $menu_itens,
            'panel_itens' => $panel_itens,
            'icon' => $xml->icon);
    }

    public function getModules()
    {
        return $this->modules;
    }

    public function getModulesName()
    {
        return $this->modulesName;
    }

    private function isoToUtf($text)
    {
        return iconv("UTF-8", "ISO-8859-1//TRANSLIT", $text);
    }

    public function scanActiveModules($active_modules)
    {
        foreach ($active_modules as $module) {
            $this->processXmlModule($module);
        }
        return $this->modules;
    }
}
