<?php

/**
 * @Company: NetPixel Tecnologia
 * @Author: fhagnerduarte
 * @Date:   2018-02-21 22:42:12
 * @Last Modified by:   fhagnerduarte
 * @Last Modified time: 2018-03-04 21:05:47
 */

namespace Framework;

class Template
{

    private $dadBlocks = array(); //armazena os blocos pais
    private $childBlocks = array(); //armazena os blocos filhos
    private $tempDadBlock = array(); //armazena temporariamente blocos para localizao do pai
    private $blocks = array(); //armazena os blocos encontrados
    private $tpl = null; // armazena o template
    private $level = 0; //nivel utilizado para montar arvore de blocos
    private $vars = array(); //varives a serem substituidas
    private $varsBlock = array(); //variveis do loop a ser substituda
    private $tempBlock = null; //armazena temporarimente um bloco para edio do seu contedo
    private $patch = 'template/'; //diretrio base dos templates
    private $select = 0;
    private $selects = array();

    /*
    Abre template principal e inicia identificao de includes e blocos dinmicos
     */
    public function __construct($page = false)
    {

        if (!$page) {
            $page = $this->patch . $this->scriptName();
        }

        $file = file($page) or die('Pgina ' . $page . ' no  um arquivo ou no foi encontrada');
        $this->tpl = implode('', $file);
        //$this->identifyIncludes($file);
        $this->identifyBlocks($file);
    }

    public function scriptName()
    {
        $page = (explode('/', $_SERVER['PHP_SELF']));
        $page = end($page);
        return str_replace('.php', '.html', $page);
        //$this->Template($page);
    }

    /*
    Associa valor do bloco a rtulos
     */
    public function __set($label, $value)
    {
        $this->varsBlock[$label] = $value;
    }

    /*
    Associa um template a um rtulo
     */
    public function assignFile($label, $page)
    {
        $file = file($this->patch . $page) or die('Pgina ' . $page . ' no  um arquivo ou no foi encontrada');
        $page = implode('', $file);

        $this->tpl = str_replace('{' . $label . '}', $page, $this->tpl);

        //$this->identifyIncludes($file);
        $this->identifyBlocks($file);
    }

    /*
    Associa valor a rtulos
     */
    public function assign($label, $value)
    {
        $this->vars[$label] = $value;
    }

    /*
    Associa valor do bloco a rtulos
     */
    public function assignBlock($label, $value)
    {
        $this->varsBlock[$label] = $value;
    }

    /*
    Varre linha por linha do template para identificar blocos
     */
    private function identifyBlocks($file)
    {
        foreach ($file as $line) {
            if (strpos($line, '<!-- include:') !== false) {
                $this->processIncludes($line);
            }

            if (strpos($line, '<!-- ') !== false) {
                $this->extractBlock($line);
            }

        }
    }

    private function processIncludes($line)
    {

        $search = '/<!-- include:(.*) -->/smi';
        if (preg_match($search, $line, $result)) {

            $includeFile = str_replace(':', '/', $result[1]);

            $file = file_get_contents($includeFile) or die('Pgina ' . $includeFile . ' no encontrada');
            $this->tpl = preg_replace('/<!-- include:' . $result[1] . ' -->/', $file, $this->tpl);

            $file = file($includeFile) or die('Pgina ' . $includeFile . ' no encontrada');
            $this->identifyBlocks($file);
        }
    }

    private function extractBlock($line)
    {
        $search = '/<!-- begin_block_(.*) -->/smi';
        if (preg_match($search, $line, $result)) {

            //monta rvore
            $this->tempDadBlock[] = $result[1];
            if ($this->level == 0) {
                //pai procura no documento inteiro
                $this->tempBlock = $this->tpl;
                $block = $this->getBlock($result[1]);
                $this->childBlocks['.'][$result[1]] = $result[1];
                $this->blocks[$result[1]] = $block;
                $this->tempBlock = $block;

                $this->dadBlocks[$result[1]] = '.';

                //substituir local
                $this->tpl = $this->clearBlock($result[1], $this->tpl);

            } else {
                $dad = $this->tempDadBlock[($this->level - 1)];
                $this->blocks[$result[1]] = $this->getBlock($result[1]);

                $this->tempBlock = $this->blocks[$dad];

                //substituir local
                $this->blocks[$dad] = $this->clearBlock($result[1], $this->blocks[$dad]);
                $this->childBlocks[$dad][$result[1]] = $result[1];
                $this->dadBlocks[$result[1]] = $dad;
            }
            $this->level++;
        }

        $search = '/<!-- end_block_(.*) -->(.*)/smi';
        if (preg_match($search, $line, $result)) {
            array_pop($this->tempDadBlock);
            $this->level--;
        }

    }

    private function getBlock($block)
    {
        //pega bloco
        $search = '/<!-- begin_block_' . $block . ' -->(.*)<!-- end_block_' . $block . ' -->/smi';
        preg_match($search, $this->tempBlock, $result) or die('Bloco ' . $block . ' no encontrado');
        return $result[1];
    }

    private function clearBlock($block, $tpl)
    {
        //apaga um bloco  colocando sua marcao
        $search = '/<!-- begin_block_' . $block . ' -->(.*)<!-- end_block_' . $block . ' -->/smi';
        return preg_replace($search, '<!-- block_' . $block . ' -->', $tpl);
    }

    public function replaceBlock($name, $replace)
    {
        $this->tpl = str_replace('<!-- block_' . $name . ' -->', $replace, $this->tpl);
    }

    private function clearChild($block)
    {
        //limpa marcaes de blocos filhos
        if (isset($this->childBlocks[$block])) {
            $children = $this->childBlocks[$block];
            foreach ($children as $child) {
                $this->tpl = str_replace('<!-- block_' . $child . ' -->', '', $this->tpl);
            }
        }
    }

    public function block($name)
    {
        //pega bloco correspodente e substitui variveis, montando a rvore
        isset($this->blocks[$name]) or die('Bloco ' . $name . ' no encontrado');
        $line = $this->blocks[$name];
        if (isset($this->varsBlock)) {
            foreach ($this->varsBlock as $index => $value) {
                $line = str_replace('{' . $index . '}', $value, $line);
            }
        }

        //limpa filhos para no haver troca em local incorreto
        $this->clearChild($name);
        $this->tpl = str_replace('<!-- block_' . $name . ' -->', $line . '<!-- block_' . $name . ' -->', $this->tpl);
        unset($this->varsBlock);
    }

    public function show()
    {
        //associa variveis
        foreach ($this->vars as $index => $value) {
            if (!is_array($value)) {
                $this->tpl = str_replace('{' . $index . '}', $value, $this->tpl);
            }

        }

        //retira finais de blocos que ficaram em aberto
        $this->tpl = preg_replace('/<!-- block_(.*) -->/', '', $this->tpl);

        print $this->tpl;
    }

    public function setPatch($patch)
    {
        $this->patch = $patch;
    }

    public function message($block, $message)
    {
        $this->tpl = str_replace('<!-- form:' . $block . ' -->', '<div class="alert alert-success"><div id="form_message"><strong>' . $message . '</strong></div></div>', $this->tpl);
    }

    public function error($block, $message)
    {
        $this->tpl = str_replace(
            '<!-- form:' . $block . ' -->',
            '<div>
                <div data-collapsed="0" class="panel panel-danger">
                    <div class="panel-heading">
                        <div class="panel-title">Erro(s) ao preencher os dados</div>
                    </div>
                    <div class="panel-body"><div id="form_error">' . $message . '</div>
                    </div>
                </div>
            </div>',
            $this->tpl);
    }

    public function assignFormVars($form_fields)
    {
        foreach ($form_fields as $field) {
            $this->assign(strtoupper($field), $_POST[$field]);
        }
    }

    public function select($block, $list, $select)
    {
        $selected = empty($_POST[$block]) ? $select : $_POST[$block];

        foreach ($list as $value => $label) {
            $this->assignBlock('SELECTED', $selected == $value ? ' selected="selected"' : null);
            $this->assignBlock('VALUE', $value);
            $this->assignBlock('LABEL', $label);
            $this->block($block);
        }
    }

    public function checkboxList($block, $list)
    {
        foreach ($list as $value => $label) {
            $this->assignBlock('CHECKED', array_search($value, $_POST[$block]) !== false ? ' checked="checked"' : null);
            $this->assignBlock('VALUE', $value);
            $this->assignBlock('LABEL', $label);
            $this->block($block);
        }
    }

    public function radio($block, $list, $id = 0)
    {

        foreach ($list as $value => $label) {
            //$this->assignBlock('CHECKED', $_POST[$block] == $value ? ' checked="checked"' : NULL);
            $this->assignBlock('CHECKED', $id == $value ? ' checked="checked"' : null);
            $this->assignBlock('VALUE', $value);
            $this->assignBlock('LABEL', $label);
            $this->block($block);
        }

    }

    public function checkbox($field)
    {
        $this->assign(strtoupper($field), $_POST[$field] == null ? null : ' checked="checked"');
    }

    public function assignSelect($block, $value, $label, $select)
    {

        $selected = empty($_POST[$block]) ? $select : $_POST[$block];

        $this->assignBlock('SELECTED', $selected == $value ? ' selected="selected"' : null);
        $this->assignBlock('VALUE', $value);
        $this->assignBlock('LABEL', $label);
        $this->block($block);
    }

    public function assignCheckbox($block, $value, $label, $selects)
    {

        $checkbox_value = is_array($_POST[$block]) ? $_POST[$block] : $selects;

        $this->assignBlock('CHECKED', array_search($value, $checkbox_value) !== false ? ' checked="checked"' : null);
        $this->assignBlock('VALUE', $value);
        $this->assignBlock('LABEL', $label);
        $this->block($block);
    }

    public function assignRadio($block, $value, $label)
    {

        $this->assignBlock('CHECKED', $_POST[$block] == $value ? ' checked="checked"' : null);
        $this->assignBlock('VALUE', $value);
        $this->assignBlock('LABEL', $label);
        $this->block($block);
    }

    public function pagePatch($list)
    {
        $last = array_pop($list);
        foreach ($list as $page => $label) {
            $this->assignBlock('BAR', '/');
            $this->assignBlock('URL', $page);
            $this->assignBlock('LABEL', $label);
            $this->block('pagepatch');
        }

        $this->assignBlock('BAR', null);
        $this->assignBlock('URL', '#');
        $this->assignBlock('LABEL', $last);
        $this->block('pagepatch');
    }

    /*
    Associa valor a rtulos
     */
    public function setText($idioma)
    {
        putenv('LANG=pt_BR');
        setlocale(LC_ALL, 'pt_BR');
        bindtextdomain($idioma, './locale/');
        textdomain($idioma);
    }

    public function getText($label)
    {
        $this->vars[$label] = _($label);
    }

}
