<?php

/**
 * @Company: NetPixel Tecnologia
 * @Author: fhagnerduarte
 * @Date:   2018-02-21 22:42:12
 * @Last Modified by:   fhagnerduarte
 * @Last Modified time: 2018-03-01 08:25:41
 */

namespace Framework;

class Validation
{

    private $vars = array();
    private $fileVars = array();
    private $errors = array();
    private $submitName = 'submit';
    private $patch;

    public function __construct($patch = 'validation/')
    {
        $this->setPatch($patch);
    }

    public function validate()
    {

        $this->vars = $_POST;
        $this->fileVars = $_FILES;

        $page = $this->patch . $this->submitName . '_validation.xml';
        file_exists($page) or die('Arquivo ' . $page . ' no encontrado');

        $xml = simplexml_load_file($page);

        foreach ($xml->children() as $field) {
            $attributes = $field->attributes();

            $field_name = (string) $attributes['name'];
            $field_value = isset($this->vars[$field_name]) ? $this->vars[$field_name] : null;

            foreach ($field->children() as $validation_type => $child) {

                $attributes = $child->attributes();
                $error = (string) $child;

                switch ($validation_type) {

                    case 'required':
                        if (!$this->required($field_value)) {
                            $this->addError($field_name, $error);
                        }

                        break;

                    case 'numeric':
                        if (!$this->isNumeric($field_valued)) {
                            $this->addError($field_name, $error);
                        }

                        break;

                    case 'date':
                        if (!$this->isDate($field_value)) {
                            $this->addError($field_name, $error);
                        }

                        break;

                    case 'email':
                        if (!$this->email($field_value)) {
                            $this->addError($field_name, $error);
                        }

                        break;

                    case 'range':
                        if (!$this->rangeValue($field_value, (string) $attributes['min'], (string) $attributes['max'])) {
                            $error = str_replace('{MIN}', (string) $attributes['min'], $error);
                            $error = str_replace('{MAX}', (string) $attributes['max'], $error);
                            $this->addError($field_name, $error);
                        }
                        break;

                    case 'compare':
                        if (!$this->compare($field_value, $this->vars[(string) $attributes['field']])) {
                            $this->addError($field_name, $error);
                        }

                        break;

                    case 'onlyone':
                        if (!$this->onlyone($field_value, $this->vars[(string) $attributes['field']])) {
                            $this->addError($field_name, $error);
                        }

                        break;

                    case 'ereg':
                        if (!$this->eregExpression($field_value, (string) $attributes['exp'])) {
                            $this->addError($field_name, $error);
                        }

                        break;

                    case 'length':
                        if (!$this->length($field_value, (string) $attributes['min'], (string) $attributes['max'])) {
                            $error = str_replace('{MIN}', (string) $attributes['min'], $error);
                            $error = str_replace('{MAX}', (string) $attributes['max'], $error);
                            $this->addError($field_name, $error);
                        }
                        break;

                    case 'integer':
                        if (!$this->isInteger($field_value)) {
                            $this->addError($field_name, $error);
                        }

                        break;

                    case 'cpf':
                        if (!$this->cpf($field_value)) {
                            $this->addError($field_name, $error);
                        }

                        break;

                    case 'cnpj':
                        if (!$this->cnpj($field_value)) {
                            $this->addError($field_name, $error);
                        }

                        break;

                    case 'filetype':
                        if (!$this->fileType($this->fileVars[$field_name], (string) $attributes['types'])) {
                            $this->addError($field_name, $error);
                        }

                        break;

                    case 'filesize':

                        if (!$this->fileSize($this->fileVars[$field_name], (string) $attributes['maxsize'])) {
                            $maxsize = number_format(((int) $attributes['maxsize'] / 1024), 2, '.', '.');
                            $error = str_replace('{MAXSIZE}', $maxsize, $error);

                            $this->addError($field_name, $error);
                        }
                        break;

                    case 'filerequired':
                        if (!$this->fileRequired($this->fileVars[$field_name])) {
                            $this->addError($field_name, $error);
                        }

                        break;

                }
            }
        }

        if (sizeof($this->errors) > 0) {
            throw new \Exception($this->getErrors(), 1);
        }
    }

    private function addError($field, $error)
    {
        if (!isset($this->errors[$field])) {
            $this->errors[$field] = $error;
        }

    }

    /* validaes */
    private function fileRequired($field)
    {
        return empty($field['name']) ? false : true;
    }

    private function fileSize($field, $maxsize)
    {

        if (!empty($field['size'])) {
            return $field['size'] > ($maxsize * 1024) ? false : true;
        }
        return true;
    }

    private function fileType($field, $types)
    {
        if (!empty($field['type'])) {
            return substr_count($types, $field['type']) == 0 ? false : true;
        }
        return true;
    }

    private function length($field, $min, $max)
    {
        if ((strlen($field) < $min) || (strlen($field) > $max)) {
            return false;
        }

        return true;
    }

    private function eregExpression($field, $exp)
    {
        if (!empty($field)) {
            // if(!eregi($exp, $field)) return false;
            if (!preg_match($exp, $field)) {
                return false;
            }

        }
        return true;
    }

    private function email($field)
    {
        if (!empty($field)) {
            if (!preg_match('/@.*.\./', $field)) {
                return false;
            }

        }
        return true;
    }

    private function compare($field, $field2)
    {
        return $field != $field2 ? false : true;
    }

    private function onlyone($field, $field2)
    {
        return !empty($field) && !empty($field2) ? false : true;
    }

    private function rangeValue($field, $min, $max)
    {
        if (($field < $min) || ($field > $max)) {
            return false;
        }

        return true;
    }

    private function required($field)
    {
        if (!is_array($field)) {
            $field = trim($field);
        }

        return empty($field) ? false : true;
    }

    private function isNumeric($field)
    {
        if (!empty($field)) {
            return is_numeric($field);
        }
        return true;
    }

    private function isInteger($field)
    {
        if (!empty($field)) {
            if (eregi("[^0-9]", $field)) {
                return false;
            }

        }
        return true;
    }

    private function isDate($field)
    {
        if (!empty($field)) {
            if (!eregi("[0-9][0-9]/[0-9][0-9]/[1-9][0-9][0-9][0-9]", $field)) {
                return false;
            }

        }
        return true;
    }

    private function cpf($cpf)
    {
        for ($i = 0; $i < 10; $i++) {
            if ($cpf == str_repeat($i, 11) or !preg_match("@^[0-9]{11}$@", $cpf) or $cpf == '12345678909') {
                return false;
            }

            if ($i < 9) {
                $soma[] = $cpf{$i} * (10 - $i);
            }

            $soma2[] = $cpf{$i} * (11 - $i);
        }
        if (((array_sum($soma) % 11) < 2 ? 0 : 11 - (array_sum($soma) % 11)) != $cpf{9}) {
            return false;
        }

        return (((array_sum($soma2) % 11) < 2 ? 0 : 11 - (array_sum($soma2) % 11)) != $cpf{10}) ? false : true;
    }

    private function cnpj($cnpj)
    {
        $cnpj = preg_replace("@[./-]@", "", $cnpj);
        if (strlen($cnpj) != 14 or !is_numeric($cnpj)) {
            return false;
        }
        $k = 6;
        $soma1 = "";
        $soma2 = "";
        for ($i = 0; $i < 13; $i++) {
            $k = $k == 1 ? 9 : $k;
            $soma2 += ($cnpj{$i} * $k);
            $k--;
            if ($i < 12) {
                if ($k == 1) {
                    $k = 9;
                    $soma1 += ($cnpj{$i} * $k);
                    $k = 1;
                } else {
                    $soma1 += ($cnpj{$i} * $k);
                }
            }
        }
        $digito1 = $soma1 % 11 < 2 ? 0 : 11 - $soma1 % 11;
        $digito2 = $soma2 % 11 < 2 ? 0 : 11 - $soma2 % 11;

        return ($cnpj{12} == $digito1 and $cnpj{13} == $digito2);
    }
    /* fim validaes */

    public function setPatch($patch)
    {
        $this->patch = $patch;
    }

    public function initializeAll($fields)
    {
        foreach ($fields as $field) {
            if (!isset($_POST[$field])) {
                $_POST[$field] = null;
            }

        }
    }

    public function initialize($field, $value)
    {
        if (!isset($_POST[$field])) {
            $_POST[$field] = $value;
        }

    }

    public function initializeToArray($fields)
    {
        foreach ($fields as $field) {
            if (!isset($_POST[$field])) {
                $_POST[$field] = array();
            }

        }
    }

    public function clear()
    {
        foreach ($this->vars as $label => $value) {
            $_POST[$label] = is_array($_POST[$label]) ? array() : null;
        }
    }

    public function send($submitName)
    {
        $this->submitName = $submitName;
        return isset($_POST[$this->submitName]);
    }

    public function cssError()
    {
        if (sizeof($this->errors) > 0) {
            $css = '<style type="text/css"> #';
            $fields = array_unique(array_keys($this->errors));
            $cssFields = implode(', #', $fields);
            $css .= $cssFields . '{ border: 1px solid #900 !important; }';
            $css .= '</style>';
            return $css;
        }
        return false;
    }

    public function getErrors()
    {
        return sizeof($this->errors) > 0 ? '<ul><li>' . implode('</li><li>', $this->errors) . '</li></ul>' : false;
    }
}
