<?php

class ControllerMonitoramento extends \Framework\Banco {

	private $path;
	private $controllerMonitoramentoAssunto = null;
	private $controllerUsuarioApp = null;
	private $controllerCidade = null;
	private $controllerLocalizacao = null;

	public function __construct() {
		parent::__construct('Monitoramento');
		$this->controllerMonitoramentoAssunto = new ControllerMonitoramentoAssunto();
		$this->controllerUsuarioApp = new ControllerUsuarioApp();
		$this->controllerCidade = new ControllerCidade();
		$this->controllerLocalizacao = new ControllerLocalizacao();
	}

	public function insert($post) {

		try {
			$obj = new Monitoramento();
			$obj->setAnonimo(isset($post['anonimo']) ? $post['anonimo'] : 'NAO');
			$obj->setImagem($post['imagem']);
			$obj->setMensagem($post['mensagem']);
			$obj->setDataCadastro(new DateTime());

			if (!empty($post['assunto'])) {
				$assunto = $this->controllerMonitoramentoAssunto->getById($post['assunto']);
				if ($assunto) {
					$obj->setAssunto($assunto);
				} else {
					throw new Exception('Não foi possível selecionar o assunto', 1);
				}
			} else {
				throw new Exception('Informe o assunto', 1);
			}

			$obj->setUsuario($this->controllerUsuarioApp->getByAttr(array('email' => $post['usuario_email'])));

			$obj->setLocalizacao($this->controllerLocalizacao->popularLocalizacao($post, new Localizacao()));

			$obj = parent::merge($obj);

			return 'Obrigado pelo seu contato';
		} catch (Exception $e) {
			throw new Exception($e->getMessage(), 1);
		}
	}

	private function montarMensagemEmail($post) {
		try {

			$assunto = $this->controllerMonitoramentoAssunto->getById($post['assunto']);

			$dados_usuario = '';

			if ($post['anonimo'] == "NAO") {
				$dados_usuario .= "<div><h4>Usuário</h4></div>
						   <div>
								<p><strong>Anônimo:</strong> {$post['anonimo']}<br/>
						<strong>Nome:</strong> {$post['usuario_nome']}<br/>
						<strong>E-mail:</strong> {$post['usuario_email']}<br/>
						<strong>Telefone:</strong> {$post['usuario_telefone']}</p>
							</div>
							<br/>";
			}

			$post['latitude'] = str_replace(',', '.', $post['latitude']);
			$post['longitude'] = str_replace(',', '.', $post['longitude']);

			$body = "<div>
						<p>Assunto</p>
						<h4>{$assunto->getTitulo()}</h4>
					</div>

					<div><p><strong>Mensagem:</strong> {$post['mensagem']}</p></div>
					<br/>

					{$dados_usuario}

					<div><h4>Localização</h4></div>
					<div>
						<p><strong>Tipo:</strong> {$post['localizacao_tipo']}<br/>
						<strong>CEP:</strong> {$post['cep']}<br/>
						<strong>Logradouro:</strong> {$post['logradouro']}<br/>
						<strong>Número:</strong> {$post['numero']}<br/>
						<strong>Bairro:</strong> {$post['bairro']}<br/>
						<strong>Cidade:</strong> {$post['cidade']}<br/>
						<strong>Estado:</strong> {$post['estado']}</p><br/>

						<p><strong>Latitude:</strong> {$post['latitude']}<br/>
						<strong>Longitude:</strong> {$post['longitude']}</p>
						<p>Google Maps Link:</b> <a href=\"https://www.google.com.br/maps/place/{$post['latitude']},{$post['longitude']} \" target=\"_blank\">MAPA</a></p>
					</div>";

			return $body;

		} catch (Exception $e) {
			throw new Exception($e->getMessage(), 1);
		}
	}

	public function enviarEmail($post) {
		try {

			$sistema = new ControllerSistema();
			$sistema = $sistema->getByAttr();

			$cidade = $this->controllerCidade->getByAttr(array('nome' => $post['cidade']));
			if ($cidade) {
				$email_monitoramento = $cidade->getEmailMonitoramento()
				? $cidade->getEmailMonitoramento()
				: $sistema->getEmailContato();
			} else {
				$email_monitoramento = $sistema->getEmailContato();
			}

			$msg = $this->montarMensagemEmail($post);

			$mail = new \Framework\Mail(
				$sistema->getSmtpHost(),
				$sistema->getSmtpUsuario(),
				$sistema->getSmtpSenha(),
				$sistema->getSmtpSeguranca(),
				$sistema->getSmtpPorta());

			if (!empty($post['imagem'])) {
				$base = base64_decode($post['imagem']);
				$mail->AddStringAttachement($base, "Filename.png", "base64", "image/png");
			}

			$mail->send(
				$email_monitoramento,
				'',
				$sistema->getEmailSender(),
				$sistema->getNome(),
				'Cadastro de monitoramento pelo aplicativo.',
				$msg);

			$mail->send(
				$post['usuario_email'],
				$post['usuario_nome'],
				$sistema->getEmailSender(),
				$sistema->getNome(),
				'Cadastro de monitoramento.',
				$msg);

			return true;
		} catch (Exception $e) {
			throw new Exception($e->getMessage(), 1);
		}
	}
}
