<?php

class ControllerNascente extends \Framework\Banco {

	private $path;
	private $controllerUsuarioApp = null;
	private $controllerFonte = null;
	private $controllerReparo = null;
	private $controllerSituacaoNascente = null;
	private $controllerTipoNascente = null;
	private $controllerIcone = null;
	private $controllerCidade = null;
	private $controllerLocalizacao = null;
	private $nascente_status = array();

	public function __construct($path = null) {
		parent::__construct('Nascente');
		$this->controllerUsuarioApp = new ControllerUsuarioApp();
		$this->controllerFonte = new ControllerFonte();
		$this->controllerReparo = new ControllerReparo();
		$this->controllerSituacaoNascente = new ControllerSituacaoNascente();
		$this->controllerTipoNascente = new ControllerTipoNascente();
		$this->controllerIcone = new ControllerIcone();
		$this->controllerCidade = new ControllerCidade();
		$this->controllerLocalizacao = new ControllerLocalizacao();

		$this->nascente_status = array(
			'PENDENTE' => 'Pendente',
			'CONFIRMADA' => 'Confirmada',
			'EXCLUIDO' => 'Excluir',
			'CIDADE_SEM_CONVENIO' => 'Cidade sem Convênio');

		$this->path = $path;
		if (!is_dir($this->path) && !is_null($this->path)) {
			mkdir($this->path, 0777);
			mkdir($this->path . "/mini/", 0777);
		}
	}

	public function getPath() {
		return $this->path;
	}

	public function getStatus() {
		return $this->nascente_status;
	}

	public function insert($post, $files = null) {

		try {
			$obj = new Nascente();
			$obj->setStatus(isset($post['status']) ? $post['status'] : 'PENDENTE');
			$obj->setOrigem(isset($post['origem']) ? $post['origem'] : 'SISTEMA');
			$obj->setTitulo($post['titulo']);
			$obj->setNumero(isset($post['numero_nascente']) ? $post['numero_nascente'] : 0);
			$obj->setSlug($this->criarSlugNascente('titulo', $post['titulo']));
			$obj->setAnonimo(isset($post['anonimo']) ? $post['anonimo'] : 'NAO');
			$obj->setObservacao($post['observacao']);
			$obj->setDescricao($post['descricao']);
			$obj->setDataCadastro( isset($post['data']) ? $post['data'] : new DateTime() );
			$obj->setDataAtualizacao(new DateTime());
			$obj->setImagemUsuario(isset($post['imagem_usuario']) ? $post['imagem_usuario'] : '');
			$obj->setVideo(isset($post['video']) ? $post['video'] : '');

			if (isset($files['imagem'])) {
				$imagem = $this->uploadImagem($files['imagem']);
				if ($imagem) {
					$obj->setImagemPrincipal($imagem);
				}
			} else {
				$obj->setImagemPrincipal('');
			}

			if (isset($files['marco'])) {
				$imagem = $this->uploadImagem($files['marco']);
				if ($imagem) {
					$obj->setMarco($imagem);
				}
			} else {
				$obj->setMarco('');
			}

			if (isset($post['usuario_email'])) {
				$obj->setUsuario(
					$this->controllerUsuarioApp->getByAttr(
						array('email' => $post['usuario_email'])));
			} else {
				$obj->setUsuario(null);
			}

			$obj->setLocalizacao($this->controllerLocalizacao->popularLocalizacao($post, new Localizacao()));

			if (!empty($post['cidade']) || !empty($post['cidade_id'])) {
				$filtro = isset($post['cidade'])
				? array('nome' => $post['cidade'], 'status' => 'ATIVO')
				: array('id' => $post['cidade_id'], 'status' => 'ATIVO');
				$cidade = $this->controllerCidade->getByAttr($filtro);
				if (!$cidade) {
					$obj->setStatus('CIDADE_SEM_CONVENIO');
				}
			}

			$obj->setCorrego(isset($post['corrego']) ? $post['corrego'] : '');
			$obj->setMicrobacia(isset($post['microbacia']) ? $post['microbacia'] : '');
			$obj->setVegetacao(isset($post['vegetacao']) ? $post['vegetacao'] : '');
			$obj->setFauna(isset($post['fauna']) ? $post['fauna'] : '');
			$obj->setAnaliseAgua(isset($post['analise_agua']) ? $post['analise_agua'] : '');
			$obj->setVazao(isset($post['vazao']) ? $post['vazao'] : '');

			if (isset($post['tipo_nascente']) && !empty($post['tipo_nascente'])) {
				$obj->setTipo($this->controllerTipoNascente->getById($post['tipo_nascente']));
			}
			if (isset($post['situacao_nascente']) && !empty($post['situacao_nascente'])) {
				$obj->setSituacao($this->controllerSituacaoNascente->getById($post['situacao_nascente']));
			}
			if (isset($post['reparo']) && !empty($post['reparo'])) {
				$obj->setReparo($this->controllerReparo->getById($post['reparo']));
			}
			if (isset($post['fonte']) && !empty($post['fonte'])) {
				$obj->setFonte($this->controllerFonte->getById($post['fonte']));
			}
			if (isset($post['icone']) && !empty($post['icone'])) {
				$obj->setIcone($this->controllerIcone->getById($post['icone']));
			}

			parent::merge($obj);

			return $obj;
		} catch (Exception $e) {
			throw new Exception($e->getMessage(), 1);
		}
	}

	public function update($post, $files, $obj) {
		try {
			$obj->setStatus(isset($post['status']) ? $post['status'] : 'PENDENTE');
			$obj->setOrigem(isset($post['origem']) ? $post['origem'] : 'SISTEMA');
			$obj->setTitulo($post['titulo']);
			$obj->setNumero(isset($post['numero_nascente']) ? $post['numero_nascente'] : 0);
			// $obj->setSlug($this->criarSlugNascente('titulo', $post['titulo'], $obj->getSlug(), $obj->getId()));
			$obj->setAnonimo(isset($post['anonimo']) ? $post['anonimo'] : 'NAO');
			$obj->setObservacao($post['observacao']);
			$obj->setDescricao($post['descricao']);
			$obj->setDataAtualizacao(new DateTime());
			$obj->setVideo(isset($post['video']) ? $post['video'] : '');

			// if (isset($files['imagem'])) {
			// 	$imagem = $this->uploadImagem($files['imagem']);
			// 	if ($imagem) {
			// 		$obj->setImagemPrincipal($imagem);
			// 	}
			// } else {
			// 	$obj->setImagemPrincipal('');
			// }

			if (isset($post['remover_imagem']) && $post['remover_imagem'] == 'S') {
                $obj->setImagemPrincipal(null);
            } else {
                $imagem = $this->uploadImagem($files['imagem']);
				if ($imagem) {
					$obj->setImagemPrincipal($imagem);
				}
            }

			// if (isset($files['marco'])) {
			// 	$imagem = $this->uploadImagem($files['marco']);
			// 	if ($imagem) {
			// 		$obj->setMarco($imagem);
			// 	}
			// } else {
			// 	$obj->setMarco('');
			// }

			 if (isset($post['remover_marco']) && $post['remover_marco'] == 'S') {
                $obj->setMarco(null);
            } else {
                $imagem = $this->uploadImagem($files['marco']);
				if ($imagem) {
					$obj->setMarco($imagem);
				}
            }

			$obj->setLocalizacao($this->controllerLocalizacao->popularLocalizacao($post, $obj->getLocalizacao()));

			$obj->setCorrego(isset($post['corrego']) ? $post['corrego'] : '');
			$obj->setMicrobacia(isset($post['microbacia']) ? $post['microbacia'] : '');
			$obj->setVegetacao(isset($post['vegetacao']) ? $post['vegetacao'] : '');
			$obj->setFauna(isset($post['fauna']) ? $post['fauna'] : '');
			$obj->setAnaliseAgua(isset($post['analise_agua']) ? $post['analise_agua'] : '');
			$obj->setVazao(isset($post['vazao']) ? $post['vazao'] : '');

			if (isset($post['tipo_nascente']) && !empty($post['tipo_nascente'])) {
				$obj->setTipo($this->controllerTipoNascente->getById($post['tipo_nascente']));
			}
			if (isset($post['situacao_nascente']) && !empty($post['situacao_nascente'])) {
				$obj->setSituacao($this->controllerSituacaoNascente->getById($post['situacao_nascente']));
			}
			if (isset($post['reparo']) && !empty($post['reparo'])) {
				$obj->setReparo($this->controllerReparo->getById($post['reparo']));
			}
			if (isset($post['fonte']) && !empty($post['fonte'])) {
				$obj->setFonte($this->controllerFonte->getById($post['fonte']));
			}
			if (isset($post['icone']) && !empty($post['icone'])) {
				$obj->setIcone($this->controllerIcone->getById($post['icone']));
			}

			parent::merge($obj);

			return $obj;
		} catch (Exception $e) {
			throw new Exception($e->getMessage(), 1);
		}
	}

	public function import($post, $files){
		try {
			$mimes = array('application/vnd.ms-excel','text/plain','text/csv','text/tsv');
			if(in_array($files['arquivo']['type'], $mimes)){
				$csv = array();
				foreach (file($files['arquivo']['tmp_name']) as $row) {
					$csv[] = str_getcsv($row, ';');
				}


				$return = array();
				$cont = 0;
				foreach ($csv as $key => $value) {
					$cont++;
					if($value[0] && $cont > 1){
						$data = new DateTime();
						if($value[5]){
							$d =  explode('/', $value[5]);
							$data->setDate($d[2], $d[1], $d[0]);
						}

						$return[] = array(
							'titulo' => 'Nascente '. $value[0],
							'numero_nascente' => $value[0],
							'observacao' => '',
							'descricao' => '',
							'cidade_id' => $post['cidade'],
							'latitude' => $value[1],
							'longitude' => $value[2],
							'bairro' => $value[4],
							'microbacia' =>$value[3],
							'data' => $data
						);
					}
				}

				foreach ($return as $key => $value) {
					$this->insert($value);
				}

			} else {
				throw new Exception('Envie um arquivo .csv', 1);
			}
		} catch (Exception $e) {
			throw new Exception($e->getMessage(), 1);
		}
	}

	private function criarSlugNascente($campo_titulo, $titulo, $slug = null, $id = null) {

		try {
			$helper = new \Framework\Helper();

			$tabela = strtolower($this->tabela);
			if ( is_null($id)) {
				$sql = "SELECT COUNT({$tabela}_id) AS total FROM {$tabela} WHERE {$campo_titulo} = :titulo ";
			} else {
				$sql = "SELECT COUNT({$tabela}_id) AS total FROM {$tabela} WHERE slug = :slug AND {$tabela}_id <> :id ";
			}

			$stmt = $this->entityManager->getConnection()->prepare($sql);

			if (is_null($id)) {
				$stmt->bindValue("titulo", $titulo);
			} else {
				$stmt->bindValue("slug", $slug);
				$stmt->bindValue("id", $id);
			}
			
			$stmt->execute();
			$lista = $stmt->fetchAll();

			$slug = $helper->createSlug($titulo);
			if (isset($lista[0]['total']) && $lista[0]['total'] > 0) {
				$slug .= '-' . ($lista[0]['total'] + 1);
			}

			return $slug;
		} catch (Exception $e) {

			throw new Exception($e->getMessage(), 1);
		}
	}

	private function uploadImagem($file) {

		try {
			if (isset($file['tmp_name']) && !empty($file['tmp_name'])) {
				$arquivo = uniqid(time()) . '.' . pathinfo($file['name'], PATHINFO_EXTENSION);

				$this->saveToFile($file, $arquivo);

				return $arquivo;
			}

			return null;
		} catch (Exception $e) {
			throw new Exception($e->getMessage(), 1);
		}
	}

	private function saveToFile($file, $nome) {
		try {
			\WideImage\WideImage::load($file['tmp_name'])
				->resize(1280, 800, 'inside')
				->saveToFile($this->path . $nome);
		} catch (Exception $e) {
			throw new Exception($e->getMessage(), 1);
		}
	}

	private function montarMensagemCadastroNascente($post) {
		try {

			$body = '';
			if ($post['anonimo'] == "NAO") {
				$body .= '<b>Informações do Usuário </b>
						   <p>
						     <b>Nome Completo:</b> ' . $post['usuario_nome'] . '<br>
						     <b>E-mail:</b> ' . $post['usuario_email'] . '<br>
						     <b>Telefone:</b> ' . $post['usuario_telefone'] . '<br>
						   </p>';
			}

			$post['latitude'] = str_replace(',', '.', $post['latitude']);
			$post['longitude'] = str_replace(',', '.', $post['longitude']);

			$body .= ' <b>Localização</b>
			   <p>
			     <b>Latitude:</b> ' . $post['latitude'] . '<br>
			     <b>Longitude:</b> ' . $post['longitude'] . '<br>
			     <b>Google Maps Link:</b> <a href="https://www.google.com.br/maps/place/' . $post['latitude'] . ',' . $post['longitude'] . '" target="_blank">https://www.google.com.br/maps/place/' . $post['latitude'] . ',' . $post['longitude'] . '</a><br>
			   </p>

			   <b>Mensagens do usuário</b>
			   <p>
			     <b>Observaçõs: </b> ' . $post['descricao'] . '<br>
			     <b>Descrição do local: </b> ' . $post['observacao'] . '<br>
			   </p>
			 ';

			return $body;

		} catch (Exception $e) {
			throw new Exception($e->getMessage(), 1);
		}
	}

	public function enviarEmail($post) {
		try {

			$sistema = new ControllerSistema();
			$sistema = $sistema->getByAttr();

			$cidade = $this->controllerCidade->getByAttr(array('nome' => $post['cidade']));
			if ($cidade) {
				$email_nascente = $cidade->getEmailNascente()
				? $cidade->getEmailNascente()
				: $sistema->getEmailContato();
			} else {
				return "Esta cidade não faz parte do convênio, mas seu cadastro foi enviado para análise";
				exit;
			}

			$msg = $this->montarMensagemCadastroNascente($post);

			$mail = new \Framework\Mail(
				$sistema->getSmtpHost(),
				$sistema->getSmtpUsuario(),
				$sistema->getSmtpSenha(),
				$sistema->getSmtpSeguranca(),
				$sistema->getSmtpPorta());

			$mail->send(
				$email_nascente,
				'',
				$sistema->getEmailSender(),
				$sistema->getNome(),
				'Cadastro de nova nascente pelo aplicativo.',
				$msg);

			$mail->send(
				$post['usuario_email'],
				$post['usuario_nome'],
				$sistema->getEmailSender(),
				$sistema->getNome(),
				'Cadastro de nova nascente.',
				$msg);

			return true;
		} catch (Exception $e) {
			throw new Exception($e->getMessage(), 1);
		}
	}

	public function latlangToDMS($lat, $lng) {
 
		$latpos = (strpos($lat, '-') !== false) ? 'S' : 'N';
		$lat = $this->decimalParaDMS($lat);

		$lngpos = (strpos($lng, '-') !== false) ? 'W' : 'E';
		$lng = $this->decimalParaDMS($lng);

		return  abs($lat['deg']) . '&deg;' . $lat['min'] . '&apos;' . $lat['sec'] . "&quot{$latpos} ". abs($lng['deg']) . '&deg;' . $lng['min'] . '&apos;' . $lng['sec'] . '&quot' . $lngpos;
	}
	private function decimalParaDMS($dec) {

		$vars = explode(".", $dec);
		$deg = $vars[0];
		$tempma = '0.' . $vars[1];

		$tempma = $tempma * 3600;
		$min = floor($tempma / 60);
		$sec = number_format( $tempma - ($min * 60), 4, '.', '');

		return array('deg' => $deg, 'min' => $min, 'sec' => $sec);
	}
}
