<?php

/**
 * @Company: NetPixel Tecnologia
 * @Author: fhagnerduarte
 * @Date:   2018-02-26 13:23:25
 * @Last Modified by:   Fhagner Duarte
 * @Last Modified time: 2019-03-18 09:40:54
 */

class ControllerNoticia extends \Framework\Banco
{

    private $path;

    public function __construct($path = null)
    {
        parent::__construct('Noticia');
        $this->path = $path;

        if (!is_dir($this->path) && !is_null($this->path)) {
            mkdir($this->path, 0777);
            mkdir($this->path . "/mini/", 0777);
        }
    }

    public function insert($post, $files)
    {

        try {
            $obj = new Noticia();
            $obj->setTitulo(($post['titulo']));
            $obj->setTexto(($post['texto']));
            $obj->setFonte(($post['fonte']));
            $obj->setStatus($post['status']);

            if (isset($post['data']) && !empty($post['data'])) {
                $data = explode('/', $post['data']);
                $obj->setData(new DateTime($data[2] . '-' . $data[1] . '-' . $data[0]));
            } else {
                $obj->setData(null);
            }

            $obj->setSlug(parent::criarSlug('titulo', $post['titulo']));

            $imagem = $this->uploadImagem($files['imagem']);
            if ($imagem) {
                $obj->setImagem($imagem);
            }

            parent::merge($obj);
            return $obj;
        } catch (Exception $e) {
            throw new Exception($e->getMessage(), 1);
        }

        return $obj;
    }

    public function update($post, $files, $obj)
    {

        try {
            $obj->setTitulo(($post['titulo']));
            $obj->setTexto(($post['texto']));
            $obj->setFonte(($post['fonte']));
            $obj->setStatus($post['status']);

            if (isset($post['data']) && !empty($post['data'])) {
                $data = explode('/', $post['data']);
                $obj->setData(new DateTime($data[2] . '-' . $data[1] . '-' . $data[0]));
            } else {
                $obj->setData(null);
            }

            $obj->setSlug(parent::criarSlug('titulo', $post['titulo'], $obj->getSlug(), $obj->getId()));

            if (isset($post['remover_imagem']) && $post['remover_imagem'] == 'S') {
                $obj->setImagem(null);
            } else {

                $imagem = $this->uploadImagem($files['imagem']);
                if ($imagem) {
                    $obj->setImagem($imagem);
                }
            }

            parent::merge($obj);
            return $obj;
        } catch (Exception $e) {
            throw new Exception($e->getMessage(), 1);
        }
    }

    private function uploadImagem($file)
    {

        try {
            if (isset($file['tmp_name']) && !empty($file['tmp_name'])) {
                $arquivo = uniqid(time()) . '.' . pathinfo($file['name'], PATHINFO_EXTENSION);

                $this->saveToFile($file, $arquivo);

                return $arquivo;
            }

            return null;
        } catch (Exception $e) {
            throw new Exception($e->getMessage(), 1);
        }
    }

    private function saveToFile($file, $nome)
    {
        try {
            \WideImage\WideImage::load($file['tmp_name'])
                ->resize(1280, 800, 'inside')
                ->saveToFile($this->path . $nome);

            \WideImage\WideImage::load($file['tmp_name'])
                ->resize(360, 260, 'outside')
                ->crop('center', 'center', 360, 260)
                ->saveToFile($this->path . 'mini/' . $nome);
        } catch (Exception $e) {
            throw new Exception($e->getMessage(), 1);
        }
    }
}
