<?php

class ControllerUsuarioApp extends \Framework\Banco {

	private $controllerLocalizacao = null;

	public function __construct($path = null) {
		parent::__construct('UsuarioApp');
		$this->controllerLocalizacao = new ControllerLocalizacao();
	}

	public function insert($post) {

		try {

			if (filter_var($post['email'], FILTER_VALIDATE_EMAIL)) {

				$obj = $this->getByAttr(array('email' => $post['email']));

				if (!$obj) {

					$obj = new UsuarioApp();
					$obj->setNome($post['nome']);
					$obj->setEmail($post['email']);
					$obj->setSenha(password_hash($post['senha'], PASSWORD_DEFAULT));
					$obj->setTelefone($post['telefone']);
					$obj->setFacebookId($post['facebook_id']);
					$obj->setGoogleUid($post['google_uid']);
					$obj->setFacebookToken($post['facebook_token']);
					$obj->setDataCadastro(new DateTime());

					$post['localizacao_tipo'] = 'MANUAL';
					$obj->setLocalizacao($this->controllerLocalizacao->popularLocalizacao($post, new Localizacao()));

					return parent::merge($obj);

				} else {
					throw new Exception('E-mail já cadastrado', 1);
				}
			} else {
				throw new Exception('Formato de e-mail incorreto', 1);
			}
		} catch (Exception $e) {
			throw new Exception($e->getMessage(), 1);
		}
	}

	public function update($post) {

		try {
			if (filter_var($post['email'], FILTER_VALIDATE_EMAIL)) {

				$obj = $this->getByAttr(array('email' => $post['email']));

				if ($obj) {

					if (!empty($post['senha'])) {
						$obj->setSenha(password_hash($post['senha'], PASSWORD_DEFAULT));
					}

					$obj->setNome($post['nome']);
					$obj->setTelefone($post['telefone']);

					$obj->setLocalizacao($this->controllerLocalizacao->popularLocalizacao($post, $obj->getLocalizacao()));

					parent::merge($obj);
					return $obj;
				}
			}

		} catch (Exception $e) {
			throw new Exception($e->getMessage(), 1);
		}
	}

	public function logar($email, $senha, $post=null) {
		try {
			if (filter_var($email, FILTER_VALIDATE_EMAIL)) {
				$obj = $this->getByAttr(array('email' => $email));
				if ($obj) {

					$usuario = array(
							'nome' => $obj->getNome(),
							'email' => $obj->getEmail(),
							'google_uid' => $obj->getGoogleUid(),
							'facebook_id' => $obj->getFacebookId(),
							'telefone' => $obj->getTelefone(),
							'cep' => $obj->getLocalizacao()->getCep(),
							'logradouro' => $obj->getLocalizacao()->getLogradouro(),
							'numero' => $obj->getLocalizacao() ? $obj->getLocalizacao()->getNumero() : '',
							'bairro' => $obj->getLocalizacao() ? $obj->getLocalizacao()->getBairro() : '',
							'cidade' => $obj->getLocalizacao()->getCidade() ? $obj->getLocalizacao()->getCidade()->getNome() : '',
							'estado' => $obj->getLocalizacao()->getCidade() ? $obj->getLocalizacao()->getCidade()->getEstado() : '',
							'complemento' => $obj->getLocalizacao() ? $obj->getLocalizacao()->getComplemento() : '');

					if (password_verify($senha, $obj->getSenha())) {
						return $usuario;
					}elseif( (isset($post['google_uid']) || isset($post['facebook_id'])) &&
								(!empty($post['google_uid']) || !empty($post['facebook_id'])) ){
						return $usuario;
					} else {
						throw new Exception('Senha incorreta', 1);
					}
				} else {
					return false;
				}
			} else {
				throw new Exception('Formato de e-mail incorreto', 1);
			}

		} catch (Exception $e) {
			throw new Exception($e->getMessage(), 1);
		}
	}

	public function solicitarSenha($email) {
		try {
			if (filter_var($email, FILTER_VALIDATE_EMAIL)) {
				$obj = parent::getByAttr(array('email' => $email));

				if ($obj) {

					$helper = new \Framework\Helper();

					$nova_senha = strtolower($helper->randomText(6));
					$hash_nova_senha = password_hash($nova_senha, PASSWORD_DEFAULT);

					$obj->setSenha($hash_nova_senha);

					parent::merge($obj);

					$this->enviarEmail($obj, $nova_senha);

					return true;
				} else {
					throw new Exception('Usuário não encontrado', 1);
				}
			} else {
				throw new Exception('Formato de e-mail incorreto', 1);
			}

		} catch (Exception $e) {
			throw new Exception($e->getMessage(), 1);
		}
	}

	public function enviarEmail($usuario, $nova_senha) {
		try {

			$sistema = new ControllerSistema();
			$sistema = $sistema->getByAttr();

			$msg = 'Prezado(a) ' . $usuario->getNome() . ', este e-mail é uma resposta a solicitação de senha de acesso realizada em ' . date('d/m/Y') . ' as ' . date('h:i');
			$msg .= '<br><br><strong>Dados para acesso:</strong><br>Senha: ' . $nova_senha;

			$mail = new \Framework\Mail(
				$sistema->getSmtpHost(),
				$sistema->getSmtpUsuario(),
				$sistema->getSmtpSenha(),
				$sistema->getSmtpSeguranca(),
				$sistema->getSmtpPorta());

			$mail->send(
				$usuario->getEmail(),
				$usuario->getNome(),
				$sistema->getEmailSender(),
				$sistema->getNome(),
				'Nova Senha Aplicativo Água para o Futuro.',
				$msg);

			return true;
		} catch (Exception $e) {
			throw new Exception($e->getMessage(), 1);
		}
	}
}
