<?php
require_once '../sistema/autoload.php';

$app = new \Slim\Slim();
$app->config('debug', getenv('DISPLAY_ERRORS'));

$app->response()->header('Access-Control-Allow-Origin', '*');
$app->response()->header('Access-Control-Allow-Methods', 'GET, POST, OPTIONS');
$app->response()->header('Content-Type', 'application/json;charset=utf-8');

$app->notFound(function () {
	echo ('Netpixel Tecnologia.');
});

$app->get('/', function () use ($app) {
	$app->status(400);
	echo ('Netpixel Tecnologia.');
	$app->stop();
});

$app->post('/TesteConexaoHostMinisterio', function () {});

$app->post('/SelecionarInfoContato', function () {
	try {
		$controllerSistema = new ControllerSistema();
		$obj = $controllerSistema->getByAttr();

		echo json_encode(
			array(
				'endereco' => $obj->getEndereco(),
				'telefone' => $obj->getTelefone(),
				'email' => $obj->getEmailContato())
		);
	} catch (Exception $e) {
		echo ('Desculpe, não foi possível selecionar os participes. Erro: ' . $e->getMessage());
	}
});

$app->post('/SelecionarCidades', function () {
	try {

		$controllerCidade = new ControllerCidade();
		$retorno = [];

		$lista = $controllerCidade->findBy(array('status' => 'ATIVO'), array('nome' => 'ASC'));
		foreach ($lista as $key => $line) {
			$retorno[] = $line->getNome();
		}

		echo json_encode($retorno);

	} catch (Exception $e) {
		echo ('Desculpe, não foi possível selecionar as cidades. Erro: ' . $e->getMessage());
	}
});

$app->post('/SelecionarParticipes', function () {
	try {

		$controllerParticipe = new ControllerParticipe();
		$retorno = [];

		$lista = $controllerParticipe->findBy(array('status' => 'ATIVO'), array('ordem' => 'DESC'));
		foreach ($lista as $key => $line) {
			$retorno[] = array(
				'nome' => $line->getNome(),
				'logo' => getenv('URL') . 'storage/participes/' . $line->getlogo());
		}

		echo json_encode($retorno);

	} catch (Exception $e) {
		echo ('Desculpe, não foi possível selecionar os participes. Erro: ' . $e->getMessage());
	}
});

$app->post('/SelecionarAssuntoMonitoramento', function () {
	try {

		$controllerMonitoramentoAssunto = new ControllerMonitoramentoAssunto();
		$assuntos = $controllerMonitoramentoAssunto->findBy(
			array('status' => 'ATIVO'), array('titulo' => 'ASC'));

		$retorno = [];
		foreach ($assuntos as $key => $assunto) {
			$retorno[] = array(
				'id' => $assunto->getId(),
				'titulo' => $assunto->getTitulo());
		}

		echo json_encode($retorno);
	} catch (Exception $e) {
		echo ('Desculpe, não foi possível selecionar os ministérios. Erro: ' . $e->getMessage());
	}
});

$app->post('/CadastrarMonitoramento', function () use ($app) {
	try {

		$request = \Slim\Slim::getInstance()->request();
		$request = json_encode($app->request()->params(), true);

		$request = json_decode($request);
		$request = json_decode($request->form, true);

		$post = [
			'imagem' => $request['imagem'],
			'assunto' => $request['assunto_id'],
			'anonimo' => !empty($request['anonimo']) ? $request['anonimo'] : 'NAO',
			'mensagem' => $request['descricao'],
			'usuario_nome' => $request['usuario']['nome'],
			'usuario_email' => $request['usuario']['email'],
			'usuario_telefone' => $request['usuario']['telefone'],
			'localizacao_tipo' => $request['localizacao']['automatica'] ? 'AUTOMATICA' : 'MANUAL',
			'latitude' => $request['localizacao']['latitude'],
			'longitude' => $request['localizacao']['longitude'],
			'cep' => $request['usuario']['endereco']['cep'],
			'logradouro' => $request['usuario']['endereco']['logradouro'],
			'numero' => $request['usuario']['endereco']['numero'],
			'bairro' => $request['usuario']['endereco']['bairro'],
			'cidade' => $request['usuario']['endereco']['cidade'],
			'estado' => $request['usuario']['endereco']['estado'],
		];

		$controllerMonitoramento = new ControllerMonitoramento();
		$msg = $controllerMonitoramento->insert($post);

		$controllerMonitoramento->enviarEmail($post);

		echo json_encode(array('resultado' => true, 'mensagem' => $msg));

	} catch (Exception $e) {
		echo json_encode(array('error' => 'Desculpe, não foi possível receber os dados do monitoramento. Erro: ' . $e->getMessage()));
	}
});

$app->post('/CadastarUsuario', function () use ($app) {
	try {

		$request = \Slim\Slim::getInstance()->request();
		$request = json_encode($app->request()->params(), true);

		$request = json_decode($request, true);
		$request = json_decode($request['usuario'], true);

		$post = [
			'google_uid' => isset($request['google_uid']) ? $request['google_uid'] : '',
			'facebook_id' => isset($request['facebook_id']) ? $request['facebook_id'] : '',
			'facebook_token' => isset($request['facebook_token']) ? $request['facebook_token'] : '',
			'nome' => $request['nome'],
			'email' => $request['email'],
			'senha' => $request['senha'],
			'telefone' => $request['telefone'],
			'endereco' => '',
			'cep' => $request['endereco']['cep'],
			'logradouro' => $request['endereco']['logradouro'],
			'numero' => $request['endereco']['numero'],
			'bairro' => $request['endereco']['bairro'],
			'cidade' => $request['endereco']['cidade'],
			'estado' => $request['endereco']['estado'],
		];

		$controllerUsuarioApp = new ControllerUsuarioApp();
		$controllerUsuarioApp->insert($post);

		$res = $controllerUsuarioApp->logar($request['email'], $request['senha']);
		echo json_encode(array('resultado' => $res), true);

	} catch (Exception $e) {
		echo json_encode(array('erro' => 'Desculpe, não foi possível receber os dados de cadastro do usuário. Erro: ' . $e->getMessage()), true);
	}
});

$app->post('/EditarUsuario', function () use ($app) {
	try {

		$request = \Slim\Slim::getInstance()->request();
		$request = json_encode($app->request()->params(), true);

		$request = json_decode($request, true);
		$request = json_decode($request['usuario'], true);

		$post = [
			'nome' => $request['nome'],
			'email' => $request['email'],
			'senha' => $request['senha'],
			'telefone' => $request['telefone'],
			'endereco' => '',
			'cep' => $request['endereco']['cep'],
			'logradouro' => $request['endereco']['logradouro'],
			'numero' => $request['endereco']['numero'],
			'bairro' => $request['endereco']['bairro'],
			'cidade' => $request['endereco']['cidade'],
			'estado' => $request['endereco']['estado'],
		];

		$controllerUsuarioApp = new ControllerUsuarioApp();
		$controllerUsuarioApp->update($post);

		echo json_encode(array('resultado' => true));

	} catch (Exception $e) {
		echo json_encode(array('error' => 'Desculpe, não foi possível receber os dados de cadastro do usuário. Erro: ' . $e->getMessage()));
	}
});

$app->post('/Logar', function () use ($app) {
	try {

		$request = \Slim\Slim::getInstance()->request();
		$request = json_encode($app->request()->params(), true);

		$request = json_decode($request, true);
		$request = json_decode($request['usuario'], true);

		$controllerUsuarioApp = new ControllerUsuarioApp();
		$res = $controllerUsuarioApp->logar($request['email'], $request['senha'], $request);

		if ($res) {
			echo json_encode(array('retorno' => $res));
		} else {
			if( isset($request['google_uid']) || isset($request['facebook_id']) ){

				$post = [
					'google_uid' => isset($request['google_uid']) ? $request['google_uid'] : '',
					'facebook_id' => isset($request['facebook_id']) ? $request['facebook_id'] : '',
					'facebook_token' => isset($request['facebook_token']) ? $request['facebook_token'] : '',
					'nome' => $request['nome'],
					'email' => $request['email'],
					'senha' => $request['senha'],
					'telefone' => $request['telefone'],
					'endereco' => '',
					'cep' => '',
					'logradouro' => '',
					'numero' => '',
					'bairro' => '',
					'cidade' => '',
					'estado' => '',
				];

				$controllerUsuarioApp = new ControllerUsuarioApp();
				$controllerUsuarioApp->insert($post);

				$res = $controllerUsuarioApp->logar($request['email'], $request['senha']);
				echo json_encode(array('retorno' => $res), true);

			}else{
				echo json_encode(array('error' => 'E-mail ou senha incorreto'));
			}
		}

	} catch (Exception $e) {
		echo json_encode(array('error' => 'Desculpe, não foi possível receber os dados de cadastro do usuário. Erro: ' . $e->getMessage()));
	}
});

$app->post('/SolicitarSenha', function () use ($app) {
	try {

		$request = \Slim\Slim::getInstance()->request();
		$request = json_encode($app->request()->params(), true);

		$request = json_decode($request, true);

		$controllerUsuarioApp = new ControllerUsuarioApp();
		$res = $controllerUsuarioApp->solicitarSenha($request['email']);

		if ($res) {
			echo json_encode(array('resultado' => 'Sua nova senha foi enviada para seu email!'));
		} else {
			echo json_encode(array('error' => 'Desculpe, não foi possível solicitar nova senha.'));
		}

	} catch (Exception $e) {
		echo json_encode(array('error' => 'Desculpe, não foi possível solicitar nova senha. Erro: ' . $e->getMessage()));
	}
});

$app->post('/CadastrarNascente', function () use ($app) {
	try {

		$request = \Slim\Slim::getInstance()->request();
		$request = json_encode($app->request()->params(), true);

		$request = json_decode($request);
		$request = json_decode($request->form, true);

		$post = [
			'imagem_usuario' => $request['imagem'],
			'origem' => 'USUARIO',
			'titulo' => 'Nova Nascente (data: ' . date('d/m/Y H:i') . ')',
			'anonimo' => $request['anonimo'] ? 'SIM' : 'NAO',
			'observacao' => $request['localizacao_descricao'],
			'descricao' => $request['descricao'],
			'usuario_nome' => $request['usuario_nome'],
			'usuario_email' => $request['usuario_email'],
			'usuario_telefone' => '',
			'localizacao_tipo' => $request['localizacao']['automatica'] ? 'AUTOMATICA' : 'MANUAL',
			'latitude' => $request['localizacao']['latitude'],
			'longitude' => $request['localizacao']['longitude'],

			'cep' => $request['localizacao']['cep'],
			// 'logradouro' => $request['localizacao']['logradouro'],
			'bairro' => $request['localizacao']['bairro'],
			'cidade' => $request['localizacao']['cidade'],
			'estado' => $request['localizacao']['estado'],
		];

		$controllerNascente = new ControllerNascente();
		$obj = $controllerNascente->insert($post);

		$resultado = $controllerNascente->enviarEmail($post);

		echo json_encode(array('resultado' => $resultado));

	} catch (Exception $e) {
		echo json_encode(array('error' => 'Desculpe, não foi possível cadastrar uma nova nascente. Erro: ' . $e->getMessage()));
	}
});

$app->post('/SelecionarNascentes', function () {
	try {
		$controllerNascente = new ControllerNascente();
		$lista = $controllerNascente->findBy(
			array('status' => 'CONFIRMADA'),
			array('numero' => 'DESC'));

		$nascentes = [];
		foreach ($lista as $key => $line) {

			$imagens = [];
			foreach ($line->getImagens() as $key => $value) {
				$imagens[] = array('url' => getenv('URL') . 'storage/nascentes/' . $value->getNome());
			}

			$video = '';
			if ($line->getVideo()) {
				$teste = explode('vimeo', $line->getVideo());
				$video = explode('/', $line->getVideo());
				if (sizeof($teste) > 1) {
					$video = 'https://player.vimeo.com/video/' . end($video);
				} else {
					$video = 'https://www.youtube.com/embed/' . end($video);
				}
			}

			$imagem = "";

			if (is_file(__DIR__ . '/../storage/nascentes/' . $line->getImagemPrincipal())) {
				$imagem = getenv('URL') . 'storage/nascentes/' . $line->getImagemPrincipal();
			} elseif (is_file(__DIR__ . '/../storage/' . $line->getImagemPrincipal())) {
				$imagem = getenv('URL') . 'storage/' . $line->getImagemPrincipal();
			}

			$texto = $line->getDescricao();
			// $texto = str_replace('src="images/', 'src="' . getenv('URL') . 'storage/images/', $texto);

			$descricao = str_replace('src="http://', 'src="https://', $line->getDescricao());
			$nascentes[] = array(
				'icone' => '',
				'usuario_email' => $line->getUsuario() ? $line->getUsuario()->getEmail() : '',
				'nascente_id' => $line->getId(),
				'municipio' => [
					'cidade' => $line->getLocalizacao()->getCidade() ? $line->getLocalizacao()->getCidade()->getNome() : '',
					'estado' => $line->getLocalizacao()->getCidade() ? $line->getLocalizacao()->getCidade()->getEstado() : ''],
				'localizacao' => ['latitude' => $line->getLocalizacao()->getLatitude(), 'longitude' => $line->getLocalizacao()->getLongitude()],
				'numero' => $line->getLocalizacao()->getNumero(),
				'titulo' => $line->getTitulo(),
				'observacao' => $line->getObservacao(),
				'descricao' => $descricao,
				'corrego' => $line->getCorrego(),
				'micro_bacia' => $line->getMicroBacia(),

				'video' => $video,
				'marco' => $line->getMarco() ? getenv('URL') . 'storage/nascentes/' . $line->getMarco() : '',

				'bairro' => $line->getLocalizacao()->getBairro(),
				'vegetacao' => $line->getVegetacao(),
				'fauna' => $line->getFauna(),
				'analise_agua' => $line->getAnaliseAgua(),
				'imagem_principal' => $imagem,
				'imagens' => $imagens,
			);

		}

		echo json_encode($nascentes);
	} catch (Exception $e) {
		echo ('Desculpe, não foi possível selecionar as Nascentes. Erro: ' . $e->getMessage());
	}
});

$app->run();
